//--------------------------------------------------------------
// File     : stm32_ub_adc1_ig.c
// Datum    : 30.10.2013
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4 
// Module   : GPIO,ADC
// Funktion : AD-Wandler (ADC1 im Injected-Group-Mode)
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_adc1_ig.h"



//--------------------------------------------------------------
// Definition der benutzten ADC Pins
// Reihenfolge wie bei ADC1i_NAME_t
//--------------------------------------------------------------
ADC1i_t ADC1i[] = {
  //NAME  ,PORT , PIN      , CLOCK              , Kanal
  {ADC_PA5,GPIOA,GPIO_Pin_5,RCC_AHB1Periph_GPIOA,ADC_Channel_5},   // ADC an PA5 = ADC12_IN5
  {ADC_PC3,GPIOC,GPIO_Pin_3,RCC_AHB1Periph_GPIOC,ADC_Channel_13},  // ADC an PC3 = ADC123_IN13
};



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_ADC1i_InitIO(void);
void P_ADC1i_InitADC(void);
void P_ADC1i_Start(void);


//--------------------------------------------------------------
// init vom ADC1 im Injected-Group-Mode
//--------------------------------------------------------------
void UB_ADC1_IG_Init(void)
{
  P_ADC1i_InitIO();
  P_ADC1i_InitADC();
  P_ADC1i_Start();
}


//--------------------------------------------------------------
// auslesen von einem ADC-Wert
//--------------------------------------------------------------
uint16_t UB_ADC1_IG_Read(ADC1i_NAME_t adc_name)
{
  // rueckgabe vom ADC-Wert

  if(adc_name==0) return ADC_GetInjectedConversionValue(ADC1,ADC_InjectedChannel_1);
  if(adc_name==1) return ADC_GetInjectedConversionValue(ADC1,ADC_InjectedChannel_2);
  if(adc_name==2) return ADC_GetInjectedConversionValue(ADC1,ADC_InjectedChannel_3);
  if(adc_name==3) return ADC_GetInjectedConversionValue(ADC1,ADC_InjectedChannel_4);

  return(0);
}


//--------------------------------------------------------------
// interne Funktion
// Init aller IO-Pins
//--------------------------------------------------------------
void P_ADC1i_InitIO(void) {
  GPIO_InitTypeDef      GPIO_InitStructure;
  ADC1i_NAME_t adc_name;

  for(adc_name=0;adc_name<ADC1i_ANZ;adc_name++) {
    // Clock Enable
    RCC_AHB1PeriphClockCmd(ADC1i[adc_name].ADC_CLK, ENABLE);

    // Config des Pins als Analog-Eingang
    GPIO_InitStructure.GPIO_Pin = ADC1i[adc_name].ADC_PIN;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
    GPIO_Init(ADC1i[adc_name].ADC_PORT, &GPIO_InitStructure);
  }
}



//--------------------------------------------------------------
// interne Funktion
// Init von ADC Nr.1
//--------------------------------------------------------------
void P_ADC1i_InitADC(void)
{
  ADC_CommonInitTypeDef ADC_CommonInitStructure;
  ADC_InitTypeDef       ADC_InitStructure;
  ADC1i_NAME_t adc_name;

  // Clock Enable
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);

  // ADC-Config
  ADC_CommonInitStructure.ADC_Mode = ADC_Mode_Independent;
  ADC_CommonInitStructure.ADC_Prescaler = ADC1i_VORTEILER;
  ADC_CommonInitStructure.ADC_DMAAccessMode = ADC_DMAAccessMode_Disabled;
  ADC_CommonInitStructure.ADC_TwoSamplingDelay = ADC_TwoSamplingDelay_5Cycles;
  ADC_CommonInit(&ADC_CommonInitStructure);

  ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b;
  ADC_InitStructure.ADC_ScanConvMode = ENABLE;
  ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
  ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  ADC_InitStructure.ADC_NbrOfConversion = 1;
  ADC_Init(ADC1, &ADC_InitStructure);

  // Anzahl der Kanle speichern
  ADC_InjectedSequencerLengthConfig(ADC1,ADC1i_ANZ);

  // hinzufuegen aller ADC-Kanle
  for(adc_name=0;adc_name<ADC1i_ANZ;adc_name++) {
    ADC_InjectedChannelConfig(ADC1,ADC1i[adc_name].ADC_CH, adc_name+1, ADC_SampleTime_3Cycles);
  }
}


//--------------------------------------------------------------
// interne Funktion
// Enable und start vom ADC1
//--------------------------------------------------------------
void P_ADC1i_Start(void)
{
  ADC_Cmd(ADC1, ENABLE);
  ADC_AutoInjectedConvCmd(ADC1,ENABLE);
  ADC_SoftwareStartConv(ADC1);
}
